// Copyright 2019 Gohilla Ltd.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/// Filter classes for complex database queries.
///
/// ```
/// import 'package:database/database.dart';
/// import 'package:database/filter.dart';
///
/// Future<List<Recipe>> findGoodItalianRecipes(Collection<Recipe> recipes) {
///   return recipes.query(
///     query: Query(
///       filter: MapFilter({
///         'rating': RangeFilter(min: 3.0),
///         'cuisine': ValueFilter('Italian'),
///       }),
///       take: 10,
///     ),
///   ).toList();
/// }
library database.filter;

export 'src/filter/filter.dart';
export 'src/filter/filter_visitor.dart';
export 'src/filter/geo_point.dart';
export 'src/filter/keyword.dart';
export 'src/filter/list.dart';
export 'src/filter/logical.dart';
export 'src/filter/map.dart';
export 'src/filter/range.dart';
export 'src/filter/range.dart';
export 'src/filter/reg_exp.dart';
export 'src/filter/sql.dart';
export 'src/filter/value.dart';
